@@ -46,7 +46,7 @@ should be the mount point, without the chroot's root prefixed.")
                                  (slot-value connection 'into))))
       ;; We only mount when the target is not already a mount point, so we
       ;; don't shadow anything that the user has already set up.
-      (unless (mountpointp dest)
+      (unless (remote-mount-point-p dest)
         (setq mount-args (copy-list mount-args))
         (setf (lastcar mount-args) dest)
         (apply #'mrun "mount" mount-args)
@@ -57,17 +57,18 @@ should be the mount point, without the chroot's root prefixed.")
     (with-slots (into) connection
       ;; Ensure the chroot itself is a mountpoint so that findmnt(8) works
       ;; correctly within the chroot.
-      (unless (mountpointp into) (chroot-mount connection "--bind" into "/"))
+      (unless (remote-mount-point-p into)
+        (chroot-mount connection "--bind" into "/"))
       ;; Now set up the usual bind mounts.  Help here from arch-chroot(8).
       (mount:assert-devtmpfs-udev-/dev)
-      (dolist (mount mount:*standard-linux-vfs*)
+      (dolist (mount mount:+linux-basic-vfs+)
         (apply #'chroot-mount connection mount))
       (chroot-mount connection "--bind" "/run" "/run")
       (when (remote-exists-p "/sys/firmware/efi/efivars")
-        (apply #'chroot-mount connection mount:*linux-efivars-vfs*)))))
+        (apply #'chroot-mount connection mount:+linux-efivars-vfs+)))))
 
 (defmethod propagate-connattr
-    ((type (eql :opened-volumes)) connattr (connection chroot-connection))
+    ((type (eql 'disk:opened-volumes)) connattr (connection chroot-connection))
   (with-slots (into) connection
     (loop for volume in connattr
           when (and (subtypep (type-of volume) 'disk:filesystem)
@@ -108,10 +109,10 @@ should be the mount point, without the chroot's root prefixed.")
     ;; ensure they don't get bogus values when this connection object is used
     ;; in UPLOAD-ALL-PREREQUISITE-DATA.
     (connection-connattr connection :remote-home)
-    ;; Obtain & cache XDG_CACHE_HOME inside the chroot, and compute DATADIR.
+    ;; Cache XDG_CACHE_HOME inside the chroot, and compute REHOME-DATADIR.
     (let ((xdg-cache-home (connection-connattr connection :XDG_CACHE_HOME)))
       (setf connection (change-class connection 'chroot.fork-connection)
-            (slot-value connection 'datadir)
+            (slot-value connection 'rehome-datadir)
             (merge-pathnames
              "consfigurator/data/" (chroot-pathname xdg-cache-home into))))
     (continue-connection connection remaining)))
@@ -119,7 +120,7 @@ should be the mount point, without the chroot's root prefixed.")
 (defmethod post-fork ((connection chroot.fork-connection))
   (with-slots (into) connection
     #+linux
-    (progn (unshare +CLONE_NEWNS+)
+    (progn (unshare CLONE_NEWNS)
            (mrun "mount" "--make-rslave"
                  (stripln (run "findmnt" "-nro" "TARGET" "-T" into)))
            (linux-chroot-mounts connection))
@@ -144,10 +145,9 @@ should be the mount point, without the chroot's root prefixed.")
 
 (defmethod connection-shell-wrap ((connection shell-chroot-connection) cmd)
   (format nil "chroot ~A sh -c ~A"
-          (escape-sh-token (unix-namestring (slot-value connection 'into)))
-          (escape-sh-token cmd)))
+          (sh-escape (slot-value connection 'into)) (sh-escape cmd)))
 
-(defmethod connection-teardown :before ((connection shell-chroot-connection))
+(defmethod connection-tear-down :before ((connection shell-chroot-connection))
   (dolist (mount (chroot-mounts connection))
     ;; There shouldn't be any processes left running in the chroot after we've
     ;; finished deploying it, but it's quite easy to end up with things like
