@@ -1,6 +1,6 @@
 ;;; Consfigurator -- Lisp declarative configuration management system
 
-;;; Copyright (C) 2021  Sean Whitton <spwhitton@spwhitton.name>
+;;; Copyright (C) 2021-2022  Sean Whitton <spwhitton@spwhitton.name>
 
 ;;; This file is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -42,13 +42,13 @@ preprocessed."
   (labels
       ((apply-*host*-propspec ()
          (let ((propapp (eval-propspec (host-propspec *host*))))
-           (assert-connection-supports (propapptype propapp))
+           (assert-connection-supports (propapp-type propapp))
            (if collect-at-end
                (let (*at-end-functions*)
-                 (let ((result (propappapply propapp)))
+                 (let ((result (apply-propapp propapp)))
                    (dolist (function *at-end-functions* result)
                      (funcall function result))))
-               (propappapply propapp))))
+               (apply-propapp propapp))))
        (connect (connections)
          (destructuring-bind ((type . args) . remaining) connections
            ;; implementations of ESTABLISH-CONNECTION which call
@@ -59,7 +59,7 @@ preprocessed."
              (if *connection*
                  (unwind-protect
                      (if remaining (connect remaining) (apply-*host*-propspec))
-                   (connection-teardown *connection*))
+                   (connection-tear-down *connection*))
                  return)))))
     (let ((*host* (preprocess-host host)))
       (cond
@@ -199,17 +199,11 @@ You can then eval (NAME) to execute this deployment."
 
 (defun hostdeploy* (host &optional additional-properties)
   "Like DEPLOY*, but use the host's default deployment."
-  (deploy* (or (host-deployment host)
-               (simple-program-error "Host has no default deployment"))
-           host
-           additional-properties))
+  (deploy* (host-deployment host) host additional-properties))
 
 (defun hostdeploy-these* (host properties)
   "Like DEPLOY-THESE*, but use the host's default deployment."
-  (deploy-these* (or (host-deployment host)
-                     (simple-program-error "Host has no default deployment"))
-                 host
-                 properties))
+  (deploy-these* (host-deployment host) host properties))
 
 (defmacro hostdeploy (host &body additional-properties)
   "Like DEPLOY, but use the host's default deployment."
@@ -244,7 +238,7 @@ for testing properties at the REPL.  See also EVALS."
                        (make-host :hostattrs `(:hostname (,,hostname))
                                   :propspec (make-propspec :systems nil)))))
        (deploy-these*
-        `((:sudo :as ,(format nil "~A@~A" ,username ,hostname)))
+        `((:sudo :from ,(format nil "~A@~A" ,username ,hostname)))
         ,host
         (let ((*host* (shallow-copy-host ,host)))
           (make-propspec :propspec (props eseqprops ,@properties)))))))
@@ -337,7 +331,7 @@ Connection attributes, by contrast, are propagated as usual."
              collect (car cell) into accum
              finally (if (eql k :data)
                          (pushnew-hostattrs :data (nreverse accum))
-                         (apply #'push-hostattrs k (nreverse accum)))))
+                         (push-hostattrs k (nreverse accum)))))
      (dolist (system (propspec-systems (host-propspec host)))
        (pushnew system (slot-value (host-propspec *host*) 'systems)))
      (setf (getf properties :host) host)))
