@@ -1,6 +1,6 @@
 ;;; Consfigurator -- Lisp declarative configuration management system
 
-;;; Copyright (C) 2017, 2021  Sean Whitton <spwhitton@spwhitton.name>
+;;; Copyright (C) 2017, 2021-2022  Sean Whitton <spwhitton@spwhitton.name>
 
 ;;; This file is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -107,7 +107,7 @@ Typically used with the ON-CHANGE combinator."
    (declare (ignore package triples))
    (os:required 'os:debianlike))
   (:apply
-   (assert-euid-root)
+   (assert-remote-euid-root)
    (run
     :input (unlines
             (loop for triple in triples collect #?"${package} @{triple}"))
@@ -169,6 +169,8 @@ EOF))
   (:unapply
    (file:does-not-exist "/etc/apt/apt.conf.d/02periodic")))
 
+;; We might add keyword arguments so that the user could disable running the
+;; cron daemon and configuring mailing root.
 (defproplist unattended-upgrades :posix ()
   "Enable unattended upgrades.
 
@@ -180,7 +182,7 @@ only upgrade Debian stable."
      (reconfigured
       "unattended-upgrades"
       '("unattended-upgrades/enable_auto_updates" "boolean" "true")))
-   (service:running "cron")
+   (service-installed-running "cron")
    (desc "unattended-upgrades will mail root"
          (file:contains-lines "/etc/apt/apt.conf.d/50unattended-upgrades"
                               "Unattended-Upgrade::Mail \"root\";"))
@@ -188,17 +190,14 @@ only upgrade Debian stable."
    (file:does-not-exist "/etc/apt/apt.conf.d/50unattended-upgrades.ucf-dist")
    :unapply (removed "unattended-upgrades")))
 
-(defprop mirror :posix (uri)
-  (:desc #?"${uri} apt mirror selected")
+(defprop mirrors :posix (&rest uris)
+  (:desc (format nil "apt mirror~P ~{~A~^, ~} selected" (length uris) uris))
   (:hostattrs
-   (pushnew-hostattr :apt.mirror uri)))
+   (pushnew-hostattrs :apt.mirrors uris)))
 
 (defpropspec uses-parent-mirrors :posix ()
   (:desc #?"Uses parent's apt mirror(s), if any")
-  (let ((mirrors (get-parent-hostattrs :apt-mirror)))
-    (and mirrors
-         `(eseqprops
-           ,@(loop for mirror in mirrors collect `(mirror ,mirror))))))
+  (aand (get-parent-hostattrs :apt.mirrors) `(mirrors ,@it)))
 
 (defprop proxy :posix (uri)
   (:desc #?"${uri} apt proxy selected")
@@ -219,15 +218,16 @@ only upgrade Debian stable."
   (proxy "http://[::1]:3142"))
 
 (defun get-mirrors ()
-  (or (get-hostattrs :apt.mirror) (call-with-os #'get-default-mirrors)))
+  (or (get-hostattrs :apt.mirrors)
+      (get-default-mirrors (get-hostattrs-car :os))))
 
 (defmethod get-default-mirrors ((os os:debian))
   '("http://deb.debian.org/debian"))
 
 (defproplist standard-sources.list :posix ()
   (:desc "Standard sources.list")
-  (file:has-content "/etc/apt/sources.list"
-    (call-with-os #'standard-sources-for)))
+  (file:has-content
+      "/etc/apt/sources.list" (standard-sources-for (get-hostattrs-car :os))))
 
 (defmethod standard-sources-for ((os os:debian))
   (let* ((suite (os:debian-suite os))
@@ -241,7 +241,7 @@ only upgrade Debian stable."
                          (mapcar (lambda (m)
                                    (list* m #?"${suite}-backports" +sections+))
                                  (get-mirrors))))
-         (security-suite (if (memstring= suite '("stretch" "jessie" "buster"))
+         (security-suite (if (memstr= suite '("stretch" "jessie" "buster"))
                              #?"${suite}/updates"
                              #?"${suite}-security"))
          (security (and (or (subtypep (type-of os) 'os:debian-stable)
@@ -283,6 +283,14 @@ after BASENAME.  CONTENT is as the content argument to FILE:HAS-CONTENT."
 
 ;;;; Pinning
 
+(defmethod suite-specifier-to-os ((suite-specifier os:debian))
+  suite-specifier)
+
+(defmethod suite-specifier-to-os ((suite-specifier cons))
+  (destructuring-bind (type &optional (suite nil suite-supplied-p))
+      suite-specifier
+    (apply #'make-instance type (and suite-supplied-p `(:suite ,suite)))))
+
 (defmethod suite-pin ((os os:debian-stable))
   (strcat "n=" (os:debian-suite os)))
 
@@ -296,22 +304,27 @@ after BASENAME.  CONTENT is as the content argument to FILE:HAS-CONTENT."
     ,(format nil "Pin-Priority: ~D" pin-priority)))
 
 (defpropspec suites-available-pinned :posix (&rest pairs)
-  "Where PAIRS is a list of even length of alternating instances of OS:DEBIAN
-and apt pin priorities, add an apt source for the instance of OS:DEBIAN and
+  "Where PAIRS is a list of even length of alternating Debian suite specifiers
+and apt pin priorities, add an apt source for the specified Debian suite and
 pin that suite to a given pin value (see apt_preferences(5)).  Unapply to drop
 the source and unpin the suite.
 
+A Debian suite specifier is either an instance of a subclass of OS:DEBIANLIKE
+or a list (TYPE &optional SUITE) where TYPE names a subclass of OS:DEBIANLIKE
+and SUITE is a string.  For example, '(os:debian-stable \"bullseye\").
+
 If the OS:DEBIAN is the host's OS, the suite is pinned, but no source is
 added.  That apt source should already be available, or you can use a property
 like APT:STANDARD-SOURCES.LIST."
   (:desc (loop for (os pin) on pairs by #'cddr
-               for suite = (os:debian-suite os)
+               for suite = (os:debian-suite (suite-specifier-to-os os))
                collect #?{Debian "${suite}" pinned, priority ${pin}}
                  into accum
                finally (return (format nil "~{~A~^; ~}" accum))))
   (:hostattrs (os:required 'os:debian))
   `(eseqprops
-    ,@(loop for (os pin) on pairs by #'cddr
+    ,@(loop for (suite-specifier pin) on pairs by #'cddr
+            for os = (suite-specifier-to-os suite-specifier)
             for suite = (os:debian-suite os)
             do (check-type pin integer)
             collect `(file:exists-with-content
@@ -333,9 +346,13 @@ like APT:STANDARD-SOURCES.LIST."
 (defpropspec pinned :posix (preferences &rest pairs)
   "Pins a list of packages, package wildcards and/or regular expressions,
 PREFERENCES, to a list of suites and corresponding pin priorities.  Unapply to
-unpin.  PAIRS is a list of even length of alternating instances of OS:DEBIAN
+unpin.  PAIRS is a list of even length of alternating Debian suite specifiers
 and apt pin priorities.
 
+A Debian suite specifier is either an instance of a subclass of OS:DEBIANLIKE
+or a list (TYPE &optional SUITE) where TYPE names a subclass of OS:DEBIANLIKE
+and SUITE is a string.  For example, '(os:debian-stable \"bullseye\").
+
 Each package, package wildcard or regular expression will be pinned to all of
 the specified suites.  Each of PREFERENCES is the name of a package, a glob to
 match the names of packages, or a regexp surrounded by slashes to match the
@@ -356,7 +373,7 @@ in testing, you could use:
                 (os:debian-testing)  100
                 (os:debian-unstable) 50)"
   (:desc (loop for (os pin) on pairs by #'cddr
-               for suite = (os:debian-suite os)
+               for suite = (os:debian-suite (suite-specifier-to-os os))
                collect #?{Debian "${suite}", priority ${pin}} into accum
                finally (return (format nil "~{~A~^, ~} pinned to ~{~A~^; ~}"
                                        preferences accum))))
@@ -366,11 +383,12 @@ in testing, you could use:
             collect (list
                      'file:exists-with-content
                      (strcat "/etc/apt/preferences.d/10consfig_"
-                             (string->filename preference)
+                             (string-to-filename preference)
                              ".pref")
                      (nbutlast
                       (loop for (os pin) on pairs by #'cddr
-                            nconc (suite-pin-block preference os pin)
+                            nconc (suite-pin-block
+                                   preference (suite-specifier-to-os os) pin)
                             collect ""))))))
 
 
