@@ -1,6 +1,6 @@
 ;;; Consfigurator -- Lisp declarative configuration management system
 
-;;; Copyright (C) 2020-2021  Sean Whitton <spwhitton@spwhitton.name>
+;;; Copyright (C) 2020-2022  Sean Whitton <spwhitton@spwhitton.name>
 
 ;;; This file is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -18,16 +18,17 @@
 (in-package :consfigurator.property.file)
 (named-readtables:in-readtable :consfigurator)
 
-(defun map-file-lines (file function)
+(defun map-remote-file-lines (file function)
   "Apply FUNCTION to the lines of FILE.  Safe to use in a :POSIX property.
 
 For efficiency, a :LISP property might want to use streams, but there's no
-point in doing that here because WRITEFILE is synchronous."
-  (let* ((orig-lines (and (remote-exists-p file) (lines (readfile file))))
+point in doing that here because WRITE-REMOTE-FILE is synchronous."
+  (let* ((orig-lines (and (remote-exists-p file)
+                          (lines (read-remote-file file))))
          (new-lines (funcall function orig-lines)))
     (if (equal orig-lines new-lines)
         :no-change
-        (writefile file (unlines new-lines)))))
+        (write-remote-file file (unlines new-lines)))))
 
 (defprop has-content :posix (path content &key (mode nil mode-supplied-p))
   "Ensure there is a file at PATH whose content is CONTENT.
@@ -36,10 +37,15 @@ CONTENT can be a list of lines or a single string."
   (:desc (declare (ignore content mode mode-supplied-p))
          #?"${path} has defined content")
   (:apply (unless mode-supplied-p (containing-directory-exists path))
-          (apply #'maybe-writefile-string
+          (apply #'maybe-write-remote-file-string
                  path
                  (etypecase content
-                   (cons (unlines content))
+                   (list
+                    ;; Within a DEFPROPSPEC it is easy to accidentally pass a
+                    ;; quoted list to this property.  Error rather than write
+                    ;; the quoted list into the remote file.
+                    (assert (every #'stringp content))
+                    (unlines content))
                    (string (format nil "~A~&" content)))
                  (and mode-supplied-p `(:mode ,mode)))))
 
@@ -66,17 +72,17 @@ replacing the contents of existing files, prefer FILE:HAS-CONTENT."
    (containing-directory-exists path)
    (let ((new-lines (copy-list (ensure-cons lines)))
          (existing-lines (and (remote-exists-p path)
-                              (lines (readfile path)))))
+                              (lines (read-remote-file path)))))
      (dolist (existing-line existing-lines)
        (deletef new-lines existing-line :test #'string=))
      (if new-lines
-         (writefile path (unlines (nconc existing-lines new-lines)))
+         (write-remote-file path (unlines (nconc existing-lines new-lines)))
          :no-change))))
 
 (defprop lacks-lines :posix (path &rest lines)
   "If there is a file at PATH, ensure it does not contain any of LINES."
   (:apply
-   (map-file-lines
+   (map-remote-file-lines
     path
     (curry #'remove-if (lambda (l) (member l lines :test #'string=))))))
 
@@ -85,7 +91,7 @@ replacing the contents of existing files, prefer FILE:HAS-CONTENT."
 any of the regular expressions PATTERNS."
   (:apply
    (let ((scanners (mapcar #'re:create-scanner patterns)))
-     (map-file-lines
+     (map-remote-file-lines
       path (lambda (lines)
              (loop for line in lines
                    unless (loop for s in scanners thereis (re:scan s line))
@@ -141,7 +147,7 @@ any of the regular expressions PATTERNS."
    (require-data iden1 iden2))
   (:apply
    (containing-directory-exists destination)
-   (maybe-writefile-data destination iden1 iden2)))
+   (maybe-write-remote-file-data destination iden1 iden2)))
 
 (defproplist host-data-uploaded :posix
     (destination
@@ -157,7 +163,7 @@ any of the regular expressions PATTERNS."
    (declare (ignore destination))
    (require-data iden1 iden2))
   (:apply
-   (maybe-writefile-data destination iden1 iden2 :mode #o600)))
+   (maybe-write-remote-file-data destination iden1 iden2 :mode #o600)))
 
 (defproplist host-secret-uploaded :posix
     (destination
@@ -176,7 +182,7 @@ any of the regular expressions PATTERNS."
   "Like s/REGEX/REPLACE/ on the lines of FILE.
 Uses CL-PPCRE:REGEX-REPLACE, which see for the syntax of REPLACE."
   (:apply
-   (map-file-lines
+   (map-remote-file-lines
     file
     (lambda (lines)
       (mapcar (lambda (line) (re:regex-replace regex line replace)) lines)))))
@@ -226,7 +232,7 @@ error if FROM is another kind of file, except when unapplying."
       "FILE:SYMLINKED: need both :FROM and :TO arguments."))
    (when (pathnamep to)
      (setq to (unix-namestring to)))
-   (let* ((link (test "-L" from))
+   (let* ((link (remote-test "-L" from))
           (exists (remote-exists-p from)))
      (when (and exists (not link))
        (failed-change "~A exists but is not a symbolic link." from))
@@ -236,7 +242,7 @@ error if FROM is another kind of file, except when unapplying."
            (containing-directory-exists from) (mrun "ln" "-sf" to from)))))
   (:unapply
    (declare (ignore to))
-   (if (test "-L" from)
+   (if (remote-test "-L" from)
        (mrun "rm" from)
        :no-change)))
 
@@ -245,7 +251,7 @@ error if FROM is another kind of file, except when unapplying."
 symbolic link, in which case the target of the link will be copied."
   (:desc #?"${dest} is copy of ${source}")
   (:check
-   (and (test "-f" dest)
+   (and (remote-test "-f" dest)
         (zerop (mrun :for-exit "cmp" "-s" dest source))))
   (:apply
    (with-remote-temporary-file
@@ -264,11 +270,11 @@ symbolic link, in which case the target of the link will be copied."
     (&key (parse-comment "#") (new-comment "# ")
        (parse-section (constantly nil)) (new-section #'identity)
        parse-kv new-kv map)
-  "Return a function suitable for passing to FILE:MAP-FILE-LINES, to modify
-the lines of a config file using MAP.  MAP is a function from a list of config
-file lines to a list of config file lines, except that lines which set values
-in the original file will be replaced by lists of the form (COMMENTED SECTION
-KEY VALUE), where
+  "Return a function suitable for passing to FILE:MAP-REMOTE-FILE-LINES, to
+modify the lines of a config file using MAP.  MAP is a function from a list of
+config file lines to a list of config file lines, except that lines which set
+values in the original file will be replaced by lists of the form (COMMENTED
+SECTION KEY VALUE), where
 
     - COMMENTED is a boolean indicating whether the line was commented
     - SECTION is the section of the config file in which the line appears
@@ -365,7 +371,7 @@ Other arguments:
                 "Values passed are not all strings, or list is not even")
           do (setf (gethash k keys) v))
     (containing-directory-exists file)
-    (map-file-lines
+    (map-remote-file-lines
      file (apply
            #'config-file-map
            :map
@@ -430,8 +436,7 @@ commented out; the first commented or uncommented line for each key will be
 uncommented and used to set the value, if it exists."
   (:desc (format nil "~A has ~{~A=~S~^, ~}" file pairs))
   (:apply (simple-conf-update file (loop for (k v) on pairs by #'cddr
-                                         collect k
-                                         collect (escape-sh-token v))
+                                         collect k collect (sh-escape v))
                               ;; include quoting as part of the value so we
                               ;; don't end up substituting double quotation
                               ;; marks for single, or similar
@@ -456,7 +461,7 @@ removed, and semicolon comment chars will be replaced with '#'."
          (keys (make-hash-table :test #'equal)))
      (loop for (s k v) in triples
            do (setf (gethash (cons s k) keys) v))
-     (map-file-lines
+     (map-remote-file-lines
       file
       (config-file-map
        :parse-comment "[#;]"
@@ -525,7 +530,7 @@ an attempt is made to activate the swap, set up the bind mount, etc."
     (dolist (entry entries)
       (setf (gethash (nth target (words entry)) pending) entry))
     (containing-directory-exists file)
-    (map-file-lines
+    (map-remote-file-lines
      file
      (lambda (lines)
        (stable-sort
