@@ -1,6 +1,6 @@
 ;;; Consfigurator -- Lisp declarative configuration management system
 
-;;; Copyright (C) 2021  Sean Whitton <spwhitton@spwhitton.name>
+;;; Copyright (C) 2021-2022  Sean Whitton <spwhitton@spwhitton.name>
 
 ;;; This file is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -67,7 +67,7 @@ Other properties might fill it in."
   (if (eql #P"/" (mount-point volume))
       1 2))
 
-(defmethod volume->entry ((volume filesystem) parent)
+(defmethod volume-to-entry ((volume filesystem) parent)
   (format nil "~A ~A ~A ~{~A~^,~} ~A ~A"
           (fs-spec volume parent) (fs-file volume)
           (fs-vfstype volume) (fs-mntops volume)
@@ -76,7 +76,7 @@ Other properties might fill it in."
 
 ;;;; Properties
 
-(defprop entries :posix (&rest entries)
+(defprop has-entries :posix (&rest entries)
   "Ensure that /etc/fstab contains each of ENTRIES, using a simple merge
 procedure: existing lines of the fstab with the same mount point as any of
 ENTRIES are updated to match the corresponding members of ENTRIES, except that
@@ -89,28 +89,34 @@ partition or filesystem UUID in your consfig."
            (format nil "fstab entr~@P for ~{~A~^, ~}" (length it) it)))
   (:apply (file:update-unix-table #P"/etc/fstab" 0 1 entries)))
 
-(defprop entries-for-volumes :posix (&optional volumes)
+(defprop has-entries-for-volumes :posix (&optional volumes)
   "Add or update entries in /etc/fstab for VOLUMES, or the host's volumes, as
 specified with DISK:HAS-VOLUMES."
   (:desc (format nil "fstab entries for ~:[~;host's ~]volumes" volumes))
   (:hostattrs (os:required 'os:linux))
-  (:apply (apply #'entries
-                 (apply #'mapcar #'volume->entry
+  (:apply (apply #'has-entries
+                 (apply #'mapcar #'volume-to-entry
                         (multiple-value-list
                          (multiple-value-mapcan
                           (curry #'subvolumes-of-type 'filesystem)
                           (or volumes (get-hostattrs :volumes))))))))
 
-(defprop entries-for-opened-volumes :posix ()
+;; TODO This is broken for fat32 partitions.  MOUNTED-FAT32-FILESYSTEM objects
+;; are pushed directly to the connattr by DISK:WITH-OPENED-VOLUMES, rather
+;; than appearing within OPENED-PARTITION objects.  Then the call to
+;; SUBVOLUMES-OF-TYPE here never finds any parents, returning as a second
+;; value a list containing only NIL.  Thus the specialisers in the
+;; implementation of FS-SPEC for MOUNTED-FAT32-FILESYSTEM are never satisfied.
+(defprop has-entries-for-opened-volumes :posix ()
   "Add or update entries in /etc/fstab for currently open volumes.
 
 This is used when building disk images and installing operating systems."
   (:desc "fstab entries for opened volumes")
   (:hostattrs (os:required 'os:linux))
   (:apply
-   (apply #'entries
-          (apply #'mapcar #'volume->entry
+   (apply #'has-entries
+          (apply #'mapcar #'volume-to-entry
                  (multiple-value-list
                   (multiple-value-mapcan
                    (curry #'subvolumes-of-type 'mounted-filesystem)
-                   (get-connattr :opened-volumes)))))))
+                   (get-connattr 'disk:opened-volumes)))))))
