@@ -1,6 +1,6 @@
 ;;; Consfigurator -- Lisp declarative configuration management system
 
-;;; Copyright (C) 2021  Sean Whitton <spwhitton@spwhitton.name>
+;;; Copyright (C) 2021-2022  Sean Whitton <spwhitton@spwhitton.name>
 
 ;;; This file is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -20,18 +20,27 @@
 
 ;;;; Bootloaders
 
-(defgeneric install-bootloader-propspec
-    (bootloader-type volume running-on-target &key)
+;;; The basic reason VOLUME-BOOTLOADERS is not just a propspec but a
+;;; specification for a propspec is that we need to pass an OPENED-VOLUME to
+;;; the property that will install the bootloader.  But maybe we could have
+;;; INSTALL-BOOTLOADER-PROPSPEC but not INSTALL-BOOTLOADER-BINARIES-PROPSPEC.
+;;; The installation of the bootloader binaries would always be done just
+;;; before installing the bootloader, and the user would not need to
+;;; separately apply BOOTLOADER-BINARIES-INSTALLED.
+
+(defgeneric install-bootloader-propspec (bootloader-type volume &key)
   (:documentation
    "Return a propspec expression which installs bootloader of type
 BOOTLOADER-TYPE to VOLUME.
 The propapp yielded by the propspec may be of type :POSIX or of type :LISP.
 
-RUNNING-ON-TARGET indicates whether the host to which we are connected is the
-host the bootloader will boot.  For example, it is NIL when building disk
-images, and T when installing a host from a live environment.  Bootloader
-installation might behave differently when RUNNING-ON-TARGET is NIL, or error
-out."))
+The property can call CONTAINER:CONTAINED-P with relevant factors to determine
+whether the host to which we are connected is the host the bootloader will
+boot.  For example, (container:contained-p :efi-nvram) returns NIL when
+building disk images, and T when installing a host from a live environment.
+Bootloader installation might behave differently when certain factors are not
+contained, or error out.  For examples, see GRUB:GRUB-INSTALLED and
+U-BOOT:INSTALLED-ROCKCHIP."))
 
 (defgeneric install-bootloader-binaries-propspec (bootloader-type volume &key)
   (:documentation
@@ -39,29 +48,29 @@ out."))
 fetches/installs whatever binaries/packages need to be available to install
 BOOTLOADER-TYPE to VOLUME."))
 
-(defun get-propspecs (volumes running-on-target)
+(defun get-propspecs (volumes)
   (loop for volume in (mapcan #'all-subvolumes volumes)
-        when (slot-boundp volume 'volume-bootloader)
-          collect (destructuring-bind (type . args) (volume-bootloader volume)
-                    (apply #'install-bootloader-propspec
-                           type volume running-on-target args))))
+        when (slot-boundp volume 'volume-bootloaders)
+          nconc (loop with bls = (volume-bootloaders volume)
+                      for bootloader in (if (listp (car bls)) bls (list bls))
+                      collect (destructuring-bind (type . args) bootloader
+                                (apply #'install-bootloader-propspec
+                                       type volume args)))))
 
 ;; At :HOSTATTRS time we don't have the OPENED-VOLUME values required by the
 ;; :APPLY subroutines which actually install the bootloaders.  So we call
-;; GET-PROPSPECS twice: (in CHROOT-INSTALLED-TO-VOLUMES) at :HOSTATTRS time to
-;; generate propspecs for the sake of running :HOSTATTRS subroutines, and then
-;; at :APPLY time where we can get at the OPENED-VOLUME values, we ignore the
-;; previously generated propspecs and call GET-PROPSPECS again.  This approach
-;; should work for any sensible VOLUME<->OPENED-VOLUME pairs.
-(define-function-property-combinator
-    %install-bootloaders (running-on-target &rest propapps)
+;; GET-PROPSPECS twice: (in CHROOT-INSTALLED-TO-VOLUMES-FOR) at :HOSTATTRS
+;; time to generate propspecs for the sake of running :HOSTATTRS subroutines,
+;; and then at :APPLY time where we can get at the OPENED-VOLUME values, we
+;; ignore the previously generated propspecs and call GET-PROPSPECS again.
+;; This approach should work for any sensible VOLUME<->OPENED-VOLUME pairs.
+(define-function-property-combinator %install-bootloaders (&rest propapps)
   (:retprop
    :type :lisp
-   :hostattrs (lambda () (mapc #'propappattrs propapps))
+   :hostattrs (lambda () (mapc #'propapp-attrs propapps))
    :apply
    (lambda ()
-     (mapc #'consfigure
-           (get-propspecs (get-connattr :opened-volumes) running-on-target))
+     (mapc #'consfigure (get-propspecs (get-connattr 'disk:opened-volumes)))
      (mrun "sync"))))
 
 
@@ -70,18 +79,17 @@ BOOTLOADER-TYPE to VOLUME."))
 (defprop %update-target-from-chroot :posix (chroot target)
   (:desc #?"Updated ${target} from ${chroot}")
   (:apply
-   (assert-euid-root)
+   (assert-remote-euid-root)
    (run "rsync" "-PSavx" "--delete"
         (loop for volume
                 in (mapcan (curry #'subvolumes-of-type 'mounted-filesystem)
-                           (get-connattr :opened-volumes))
+                           (get-connattr 'disk:opened-volumes))
               collect (strcat "--exclude="
                               (unix-namestring (mount-point volume))))
         (strcat (unix-namestring chroot) "/")
         (strcat (unix-namestring target) "/"))))
 
-(defpropspec chroot-installed-to-volumes :lisp
-    (host chroot volumes &key running-on-target)
+(defpropspec chroot-installed-to-volumes-for :lisp (host chroot volumes)
   "Where CHROOT contains the root filesystem of HOST and VOLUMES is a list of
 volumes, recursively open the volumes and rsync in the contents of CHROOT.
 Also update the fstab and crypttab, and try to install bootloader(s)."
@@ -92,7 +100,7 @@ Also update the fstab and crypttab, and try to install bootloader(s)."
             (drop-trailing-slash
              (unix-namestring (ensure-directory-pathname chroot)))
             ".target"))))
-    `(with-these-open-volumes (,volumes :mount-below ,target)
+    `(with-opened-volumes (,volumes :mount-below ,target)
        (%update-target-from-chroot ,chroot ,target)
        (chroot:deploys-these
         ,target ,host
@@ -111,11 +119,10 @@ Also update the fstab and crypttab, and try to install bootloader(s)."
                    ;; from the volumes the user has declared for the host, it
                    ;; is unlikely there are other properties setting mount
                    ;; options etc. which are in conflict with VOLUMES.
-                   (fstab:entries-for-opened-volumes)
-                   (crypttab:entries-for-opened-volumes))))
+                   (fstab:has-entries-for-opened-volumes)
+                   (crypttab:has-entries-for-opened-volumes))))
             (%install-bootloaders
-             ,running-on-target
-             ,@(get-propspecs (get-hostattrs :volumes) running-on-target))))))))
+             ,@(get-propspecs (get-hostattrs :volumes)))))))))
 
 (defpropspec bootloader-binaries-installed :posix ()
   "Install whatever binaries/packages need to be available to install the host's
@@ -124,23 +131,26 @@ install a package providing /usr/sbin/grub-install, but it won't execute it."
   (:desc "Bootloader binaries installed")
   (loop
     for volume in (mapcan #'all-subvolumes (get-hostattrs :volumes))
-    when (slot-boundp volume 'volume-bootloader)
-      collect (destructuring-bind (type . args) (volume-bootloader volume)
-                (apply #'install-bootloader-binaries-propspec type volume args))
+    when (slot-boundp volume 'volume-bootloaders)
+      nconc (loop with bls = (volume-bootloaders volume)
+                  for bootloader in (if (listp (car bls)) bls (list bls))
+                  collect (destructuring-bind (type . args) bootloader
+                            (apply #'install-bootloader-binaries-propspec
+                                   type volume args)))
         into propspecs
     finally
        (setq propspecs (delete-duplicates propspecs :test #'tree-equal))
        (return
          (if (cdr propspecs) (cons 'eseqprops propspecs) (car propspecs)))))
 
-(defpropspec bootloaders-installed :lisp (&key (running-on-target t))
+(defpropspec bootloaders-installed :lisp ()
   "Install the host's bootloaders to its volumes.
 Intended to be attached to properties like INSTALLER:CLEANLY-INSTALLED-ONCE
 using a combinator like ON-CHANGE, or applied manually with DEPLOY-THESE."
   (:desc "Bootloaders installed")
   `(eseqprops
     (bootloader-binaries-installed)
-    ,@(get-propspecs (get-hostattrs :volumes) running-on-target)))
+    ,@(get-propspecs (get-hostattrs :volumes))))
 
 
 ;;;; Live replacement of GNU/Linux distributions
@@ -160,16 +170,16 @@ using a combinator like ON-CHANGE, or applied manually with DEPLOY-THESE."
 ;;; the definition of SERVICES:WITHOUT-STARTING-SERVICES to *AT-END-FUNCTIONS*
 ;;; in a closure.  We'd also want %CONSFIGURE to use UNWIND-PROTECT to ensure
 ;;; that the AT-END functions get run even when there's a nonlocal exit from
-;;; %CONSFIGURE's call to PROPAPPAPPLY; perhaps we could pass a second
+;;; %CONSFIGURE's call to APPLY-PROPAPP; perhaps we could pass a second
 ;;; argument to the AT-END functions indicating whether there was a non-local
-;;; transfer of control.  REBOOT:REBOOTED-AT-END might only reboot when there
-;;; was a normal return from PROPAPPAPPLY, whereas the cleanup forms from
+;;; transfer of control.  REBOOT:AT-END might only reboot when there was a
+;;; normal return from APPLY-PROPAPP, whereas the cleanup forms from
 ;;; SERVICES:WITHOUT-STARTING-SERVICES would always be evaluated.
 
 (defprop %root-filesystems-flipped :lisp (new-os old-os)
   (:hostattrs (os:required 'os:linux))
   (:apply
-   (assert-euid-root)
+   (assert-remote-euid-root)
    (let ((new-os (ensure-directory-pathname new-os))
          (old-os
            (ensure-directories-exist (ensure-directory-pathname old-os)))
@@ -186,10 +196,16 @@ using a combinator like ON-CHANGE, or applied manually with DEPLOY-THESE."
              ;; This we make use of below.
              #P"/old-run/"))
          efi-system-partition-mount-args)
-     (flet ((preservedp (pathname)
+     (flet ((system (&rest args)
+              (alet (loop for arg in args
+                          if (pathnamep arg)
+                            collect (unix-namestring arg)
+                          else collect arg)
+                (foreign-funcall "system" :string (sh-escape it) :int)))
+            (preservedp (pathname)
               (member pathname preserved-directories :test #'pathname-equal)))
        (mount:assert-devtmpfs-udev-/dev)
-       (unless (mountpointp "/run")
+       (unless (remote-mount-point-p "/run")
          (failed-change "/run is not a mount point; don't know what to do."))
 
        ;; If there's an EFI system partition, we need to store knowledge of
@@ -198,7 +214,7 @@ using a combinator like ON-CHANGE, or applied manually with DEPLOY-THESE."
        ;; is responsible for adding an entry for the EFI system partition to
        ;; the new system's fstab, but we are responsible for restoring
        ;; knowledge of the partition to the kernel's mount table.
-       (when (mountpointp "/boot/efi")
+       (when (remote-mount-point-p "/boot/efi")
          (destructuring-bind (type source options)
              (words (stripln (run "findmnt" "-nro" "FSTYPE,SOURCE,OPTIONS"
                                   "/boot/efi")))
@@ -237,12 +253,12 @@ using a combinator like ON-CHANGE, or applied manually with DEPLOY-THESE."
        (let (done)
          (handler-case
              (flet ((rename (s d) (rename-file s d) (push (cons s d) done)))
-               (dolist (file (directory-contents #P"/"))
+               (dolist (file (local-directory-contents #P"/"))
                  (unless (or (preservedp file)
                              (pathname-equal file new-os)
                              (pathname-equal file old-os))
                    (rename file (chroot-pathname file old-os))))
-               (dolist (file (directory-contents new-os))
+               (dolist (file (local-directory-contents new-os))
                  (let ((dest (in-chroot-pathname file new-os)))
                    (unless (preservedp dest)
                      (when (or (file-exists-p dest) (directory-exists-p dest))
@@ -279,7 +295,7 @@ using a combinator like ON-CHANGE, or applied manually with DEPLOY-THESE."
 
        ;; Remount (mainly virtual) filesystems that other properties we will
        ;; apply might require (esp. relevant for installing bootloaders).
-       (dolist (mount mount:*standard-linux-vfs*)
+       (dolist (mount mount:+linux-basic-vfs+)
          (unless (preservedp (ensure-directory-pathname (lastcar mount)))
            (apply #'system "mount" mount)))
        (when efi-system-partition-mount-args
@@ -287,18 +303,17 @@ using a combinator like ON-CHANGE, or applied manually with DEPLOY-THESE."
          (apply #'mrun "mount" efi-system-partition-mount-args))))))
 
 (defproplist cleanly-installed-once :lisp
-    (original-os-architecture
-     &optional options
-     &aux (minimal-new-host
-	   (make-host :hostattrs (list :os (get-hostattrs :os))))
-     (original-host
-      (make-host
-       :propspec
-       (make-propspec
-        :propspec
-        `(eseqprops
-          (os:linux ,original-os-architecture)
-          (chroot:os-bootstrapped-for ,options "/new-os" ,minimal-new-host))))))
+    (&optional options original-os
+               &aux
+               (new (make-host :hostattrs `(:os ,(get-hostattrs :os))))
+               (original-host
+                (make-host
+                 :propspec
+                 (make-propspec
+                  :propspec
+                  `(eseqprops
+                    ,(or original-os '(os:linux))
+                    (chroot:os-bootstrapped-for ,options "/new-os" ,new))))))
   "Replaces whatever operating system the host has with a clean installation of
 the OS that the host is meant to have, and reboot, once.  This is intended for
 freshly launched machines in faraway datacentres, where your provider has
@@ -310,17 +325,19 @@ but not captured by your consfig.  This property's approach can fail and leave
 the system unbootable, but it's an time-efficient way to ensure that you're
 starting from a truly clean slate for those cases in which it works.
 
-ORIGINAL-OS-ARCHITECTURE is the architecture of the original OS as would be
-supplied to the OS:LINUX property, e.g. :AMD64.  OPTIONS will be passed on to
-CHROOT:OS-BOOTSTRAPPED-FOR, which see.
+OPTIONS will be passed on to CHROOT:OS-BOOTSTRAPPED-FOR, which see.
+ORIGINAL-OS, if supplied, is a propapp specifying the old OS, as you would
+apply to a host with that OS.
 
-The internal property CHROOT::%OS-BOOTSTRAPPER-INSTALLED will attempt to use
-PACKAGE:INSTALLED to install the OS bootstrapper (e.g. debootstrap(8) for
-Debian).  Alternatively, you can install the bootstrapper manually before
-running Consfigurator; this is useful for original OSs whose package managers
-Consfigurator doesn't yet know how to drive.  You might apply an OS-agnostic
-property before this one which manually downloads the bootstrapper and puts it
-on PATH.
+The internal property CHROOT::%OS-BOOTSTRAPPER-INSTALLED will attempt to
+install the OS bootstrapper (e.g. debootstrap(8) for Debian).  If ORIGINAL-OS
+is supplied then installation will use a package manager property for that OS.
+Otherwise, CHROOT::%OS-BOOTSTRAPPER-INSTALLED will fall back to trying
+PACKAGE:INSTALLED.  Alternatively, you can install the bootstrapper manually
+before running Consfigurator and not supply ORIGINAL-OS.  This is useful for
+original OSs whose package managers Consfigurator doesn't yet know how to
+drive.  You might apply an OS-agnostic property before this one which manually
+downloads the bootstrapper and puts it on PATH.
 
 The files from the old OS will be left in '/old-os'.  Typically you will need
 to perform some additional configuration before rebooting to increase the
@@ -347,7 +364,7 @@ Here are some other propapps you might want to attach to the application of
 this property with ON-CHANGE:
 
     (bootloaders-installed)
-    (fstab:entries-for-volumes
+    (fstab:has-entries-for-volumes
      (disk:volumes
        (mounted-ext4-filesystem :mount-point #P\"/\")
        (partition (mounted-fat32-filesystem :mount-point #P\"/boot/efi/\"))))
@@ -371,7 +388,7 @@ of properties subsequent to this one.  A more complete example:
       (apt:installed \"linux-image-amd64\")
       (installer:bootloaders-installed)
 
-      (fstab:entries-for-volumes
+      (fstab:has-entries-for-volumes
        (disk:volumes
          (mounted-ext4-filesystem :mount-point #P\"/\")
          (partition
@@ -394,4 +411,4 @@ live system and use Consfigurator to install to the host's usual storage."
     (%root-filesystems-flipped "/new-os" "/old-os")
     ;; Prevent boot issues caused by disabled shadow passwords.
     (cmd:single "shadowconfig" "on")
-    (reboot:rebooted-at-end)))
+    (reboot:at-end)))
