@@ -68,7 +68,7 @@ On Debian, it is not started by default after installation of libvirt."
 
 (defmethod os-variant (os))
 
-(defprop defined :posix (host &rest arguments)
+(defprop defined-for :posix (host &rest arguments)
   "Define a libvirt domain for HOST by providing ARGUMENTS to virt-install(1).
 With the current implementation, if ARGUMENTS changes, virt-install(1) will
 not be run again.  You will need to either unapply and reapply this property,
@@ -87,7 +87,7 @@ subcommand of virsh(1) to convert the running domain into a transient domain."
        nil
        "virt-install --print-xml -n ~A~:[~; --os-variant=~:*~A~]~{ ~A~} >~S"
        (get-hostname host) (os-variant host)
-       (mapcar #'escape-sh-token arguments) file))
+       (mapcar #'sh-escape arguments) file))
      (mrun "virsh" "define" file)))
   (:unapply
    (declare (ignore arguments))
@@ -120,13 +120,13 @@ already running, for a VM which is not always booted, e.g. on a laptop."
   (macrolet ((check-started (form)
                `(if (host-domain-started-p host)
                     ,form :no-change)))
-    (:retprop :type (propapptype propapp)
+    (:retprop :type (propapp-type propapp)
               :desc (get (car propapp) 'desc)
               :hostattrs (get (car propapp) 'hostattrs)
               :apply (lambda-ignoring-args
-                       (check-started (propappapply propapp)))
+                       (check-started (apply-propapp propapp)))
               :unapply (lambda-ignoring-args
-                         (check-started (propappunapply propapp)))
+                         (check-started (unapply-propapp propapp)))
               :args (cdr propapp))))
 
 ;; Another possible approach would be to convert DISK:VOLUME values to --disk
@@ -195,7 +195,7 @@ OPTIONS is a plist of keyword parameters.
   - :APPEND -- String to append to the kernel command line.
 
 If the :VCPUS, :MEMORY, :AUTOSTART or :VIRT-OPTIONS parameters change,
-virt-install(1) will not be rerun; see LIBVIRT:DEFINED.
+virt-install(1) will not be rerun; see LIBVIRT:DEFINED-FOR.
 
 Sample usage:
 
@@ -283,7 +283,7 @@ your preferred VM networking setup and corresponding DEPLOYS propapp."
              `((with-flagfile ,flagfile
                  (chroot:os-bootstrapped-for
                   ,chroot-options ,chroot ,host ,additional-properties))))
-       (defined ,host*
+       (defined-for ,host*
            ,(format nil "--vcpus=~D" vcpus) ,(format nil "--memory=~D" memory)
          "--filesystem"
          ,(format
@@ -303,7 +303,7 @@ your preferred VM networking setup and corresponding DEPLOYS propapp."
        ,@(and autostart `((started ,host)))
        :unapply
        (destroyed ,host*)
-       (unapplied (defined ,host*))
+       (unapplied (defined-for ,host*))
        (unapplied
         (with-flagfile ,flagfile
           (chroot:os-bootstrapped-for
