@@ -52,7 +52,7 @@ exist, so that the user can easily override this default."
                           (debianlike (apt:installed "eatmydata" "ccache"))))))))
              (os (get-hostattrs-car :os host))
              (suite (os:debian-suite os))
-             (arch (os:debian-architecture os)))
+             (arch (os:debian-architecture-string os)))
   "Build and configure a schroot for use with sbuild.
 For convenience we set up several enhancements, such as ccache and eatmydata.
 In the case of Debian, we assume you are building for Debian stretch or newer,
@@ -73,7 +73,7 @@ Example usage:
 
     (os:debian-stable \"bullseye\" :amd64)
     (apt:uses-local-cacher)
-    (apt:mirror \"...\")
+    (apt:mirrors \"...\")
     (sbuild:usable-by \"spwhitton\")
     (schroot:overlays-in-tmpfs)
     (periodic:at-most :monthly \"sbuild sid schroot rebuilt\"
@@ -117,7 +117,7 @@ To take advantage of the piuparts and autopkgtest support, add to your
 
        ;; ccache
        ,@(and use-ccache '((%sbuild-ccache-has-some-limits)
-                           (ccache:group-cache "sbuild")))
+                           (ccache:cache-for-group "sbuild")))
        (desc
         "ccache mounted in sbuild schroots"
         (file:contains-lines "/etc/schroot/sbuild/fstab"
@@ -156,8 +156,9 @@ EOF :mode #o755)
               ;; to avoid more than one schroot getting the same aliases, we
               ;; only do this if the arch of the chroot equals the host arch.
               ,@(and (string= suite "unstable")
-                     (string= arch (os:debian-architecture
-                                    (get-hostattrs-car :os)))
+                     (string=
+                      arch
+                      (os:debian-architecture-string (get-hostattrs-car :os)))
                      `(("aliases"
                         ,(format
                           nil "~@{~A~^,~}"
@@ -172,7 +173,7 @@ EOF :mode #o755)
                           "UNRELEASED"
                           ;; The following is for dgit compatibility.
                           (strcat "UNRELEASED-"
-                                  (os:debian-architecture os)
+                                  (os:debian-architecture-string os)
                                   "-sbuild")))))
 
               ("command-prefix"
@@ -187,12 +188,12 @@ EOF :mode #o755)
        (file:does-not-exist ,conf))))
 
 ;; Here we combine Propellor's Sbuild.osDebianStandard and Sbuild.update.
-(defpropspec standard-debian-schroot :posix (&optional (period :weekly))
+(defpropspec standard-debian-schroot :posix (&key (upgrade :weekly))
   "Properties that will be wanted in almost any Debian sbuild schroot, but not
 in sbuild schroots for other operating systems.
 
 Includes replacing use of sbuild-update(1)."
   (:desc "Standard Debian sbuild properties")
   `(eseqprops (apt:standard-sources.list)
-              (periodic:at-most ,period "sbuild schroot updated"
+              (periodic:at-most ,upgrade "sbuild schroot updated"
                 (apt:updated) (apt:upgraded) (apt:autoremoved))))
